extern crate proc_macro2;
extern crate quote;
extern crate syn;

extern crate proc_macro;

use proc_macro::TokenStream; // 用于处理宏输入
use quote::quote; // 用于生成代码
use syn::{Data, DeriveInput, Fields}; // 用于解析宏输入

pub fn gen_responsable(input: DeriveInput) -> TokenStream {
    let name = &input.ident; // 获取结构体名称
    let generics = &input.generics; // 获取泛型参数
    let (impl_generics, _ty_generics, where_clause) = generics.split_for_impl();
    let gen = match input.data {
        Data::Struct(ref data) => match data.fields {
            Fields::Named(ref _fields) => {
                // 处理命名字段

                // 生成实现`IntoResponse` trait的代码
                quote! {
                    impl #impl_generics axum::response::IntoResponse for #name #generics #where_clause {
                        fn into_response(self) -> axum::http::Response<axum::body::Body> {
                            let mut builder = hyper::Response::builder();
                            let body = serde_json::to_string(&self).expect("Failed to serialize response"); // 序列化结构体为JSON字符串
                            axum::http::Response::new(axum::body::Body::from(body)) // 构建响应
                        }
                    }
                }
            },
            Fields::Unnamed(ref _fields) => {
                // 处理未命名字段

                // 生成实现`IntoResponse` trait的代码
                quote! {
                    impl #impl_generics axum::response::IntoResponse for #name #generics #where_clause {
                        fn into_response(self) -> axum::http::Response<axum::body::Body> {
                            let body = serde_json::to_string(&self).expect("Failed to serialize response"); // 序列化结构体为JSON字符串
                            axum::http::Response::new(axum::body::Body::from(body)) // 构建响应
                        }
                    }
                }
            },
            Fields::Unit => {
                // 处理无字段的结构体

                // 生成实现`IntoResponse` trait的代码
                quote! {
                    impl #impl_generics axum::response::IntoResponse for #name #generics #where_clause {
                        fn into_response(self) -> axum::http::Response<axum::body::Body> {
                            let body = serde_json::to_string(&self).expect("Failed to serialize response"); // 序列化结构体为JSON字符串
                            axum::http::Response::new(hyper::body::Body::from(body)) // 构建响应
                        }
                    }
                }
            },
        },
        Data::Enum(_) | Data::Union(_) => {
            // 不支持枚举或联合体
            unimplemented!("IntoResponse derive is not implemented for enums or unions.")
        },
    };
    gen.into()
}